@@ -1,14 +1,23 @@
 pub use crate::x64_decl::{GPR, XMM};
 use dynasm::dynasm;
-use dynasmrt::{x64::Assembler, AssemblyOffset, DynamicLabel, DynasmApi, DynasmLabelApi};
+use dynasmrt::{
+    x64::X64Relocation, AssemblyOffset, DynamicLabel, DynasmApi, DynasmLabelApi, VecAssembler,
+};
 
-/// Dynasm proc-macro checks for an `.arch` expression in a source file to
-/// determine the architecture it should use.
-fn _dummy(_a: &Assembler) {
-    dynasm!(
-        _a
-        ; .arch x64
-    );
+type Assembler = VecAssembler<X64Relocation>;
+
+/// Force `dynasm!` to use the correct arch (x64) when cross-compiling.
+/// `dynasm!` proc-macro tries to auto-detect it by default by looking at the
+/// `target_arch`, but it sees the `target_arch` of the proc-macro itself, which
+/// is always equal to host, even when cross-compiling.
+macro_rules! dynasm {
+    ($a:expr ; $($tt:tt)*) => {
+        dynasm::dynasm!(
+            $a
+            ; .arch x64
+            ; $($tt)*
+        )
+    };
 }
 
 #[derive(Copy, Clone, Debug, Eq, PartialEq, Ord, PartialOrd, Hash)]
@@ -38,6 +47,7 @@ pub enum Condition {
     NotEqual,
     Signed,
     Carry,
+    Overflow,
 }
 
 #[derive(Copy, Clone, Debug, Eq, PartialEq, Ord, PartialOrd)]
@@ -474,7 +484,7 @@ macro_rules! binop_shift {
 
 macro_rules! jmp_op {
     ($ins:ident, $assembler:tt, $label:ident) => {
-        dynasm!($assembler ; $ins =>$label);
+        dynasm!($assembler ; $ins =>$label)
     }
 }
 
@@ -840,6 +850,7 @@ impl Emitter for Assembler {
             Condition::NotEqual => jmp_op!(jne, self, label),
             Condition::Signed => jmp_op!(js, self, label),
             Condition::Carry => jmp_op!(jc, self, label),
+            Condition::Overflow => jmp_op!(jo, self, label),
         }
     }
     fn emit_jmp_location(&mut self, loc: Location) {
@@ -863,6 +874,7 @@ impl Emitter for Assembler {
             Condition::NotEqual => dynasm!(self ; setne Rb(dst as u8)),
             Condition::Signed => dynasm!(self ; sets Rb(dst as u8)),
             Condition::Carry => dynasm!(self ; setc Rb(dst as u8)),
+            Condition::Overflow => dynasm!(self ; seto Rb(dst as u8)),
             _ => panic!("singlepass can't emit SET {:?} {:?}", condition, dst),
         }
     }
@@ -1400,7 +1412,7 @@ impl Emitter for Assembler {
     }
 
     fn emit_bkpt(&mut self) {
-        dynasm!(self ; int 0x3);
+        dynasm!(self ; int3);
     }
 
     fn emit_host_redirection(&mut self, target: GPR) {
